@@ -24,6 +24,7 @@ from awlsim.core.compat import *
 
 from awlsim.core.datatypes import *
 from awlsim.core.statusword import *
+from awlsim.core.lstack import *
 from awlsim.core.util import *
 
 
@@ -51,12 +52,6 @@ class AwlOperator(object):
 	MEM_PA		= EnumGen.item	# Peripheral output
 	MEM_PE		= EnumGen.item	# Peripheral input
 
-	MEM_DBLG	= EnumGen.item	# DB-register: DB length
-	MEM_DBNO	= EnumGen.item	# DB-register: DB number
-	MEM_DILG	= EnumGen.item	# DI-register: DB length
-	MEM_DINO	= EnumGen.item	# DI-register: DB number
-	MEM_AR2		= EnumGen.item	# AR2 register
-
 	MEM_STW		= EnumGen.item	# Status word bit read
 	MEM_STW_Z	= EnumGen.item	# Status word "==0" read
 	MEM_STW_NZ	= EnumGen.item	# Status word "<>0" read
@@ -66,7 +61,12 @@ class AwlOperator(object):
 	MEM_STW_NEGZ	= EnumGen.item	# Status word "<=0" read
 	MEM_STW_UO	= EnumGen.item	# Status word "UO" read
 
-	LBL_REF		= EnumGen.item	# Label reference
+	MEM_DBLG	= EnumGen.item	# DB-register: DB length
+	MEM_DBNO	= EnumGen.item	# DB-register: DB number
+	MEM_DILG	= EnumGen.item	# DI-register: DB length
+	MEM_DINO	= EnumGen.item	# DI-register: DB number
+
+	MEM_AR2		= EnumGen.item	# AR2 register
 
 	BLKREF_FC	= EnumGen.item	# FC reference
 	BLKREF_SFC	= EnumGen.item	# SFC reference
@@ -78,15 +78,16 @@ class AwlOperator(object):
 	BLKREF_OB	= EnumGen.item	# OB reference (only symbol table)
 	BLKREF_VAT	= EnumGen.item	# VAT reference (only symbol table)
 
+	LBL_REF		= EnumGen.item	# Label reference
 	SYMBOLIC	= EnumGen.item	# Classic symbolic reference ("xyz")
 	NAMED_LOCAL	= EnumGen.item	# Named local reference (#abc)
 	NAMED_LOCAL_PTR	= EnumGen.item	# Pointer to named local (P##abc)
-	INTERF_DB	= EnumGen.item	# Interface-DB access (translated NAMED_LOCAL)
 
 	INDIRECT	= EnumGen.item	# Indirect access
 	UNSPEC		= EnumGen.item	# Not (yet) specified memory region
 
-	# Virtual operators used for debugging of the simulator
+	# Virtual operators used internally in awlsim, only.
+	# These operators do not have standard AWL mnemonics.
 	VIRT_ACCU	= EnumGen.item	# Accu
 	VIRT_AR		= EnumGen.item	# AR
 	VIRT_DBR	= EnumGen.item	# DB and DI registers
@@ -144,21 +145,26 @@ class AwlOperator(object):
 		BLKREF_VAT	: "BLOCK_VAT",
 
 		NAMED_LOCAL	: "#LOCAL",
-		INTERF_DB	: "__INTERFACE_DB",
 
 		INDIRECT	: "__INDIRECT",
 
 		VIRT_ACCU	: "__ACCU",
 		VIRT_AR		: "__AR",
+		VIRT_DBR	: "__DBR",
 	}
 
 	def __init__(self, type, width, value, insn=None):
+		# type -> The operator type ID number. See "Operator types" above.
+		# width -> The bit width of the access.
+		# value -> The value. May be an AwlOffset or a string (depends on type).
+		# insn -> The instruction this operator is used in. May be None.
 		self.type = type
 		self.width = width
 		self.value = value
 		self.labelIndex = None
 		self.insn = insn
 		self.isExtended = False
+		# The symbol resolver may also put a "self.interfaceIndex" number in here.
 
 	# Make a deep copy, except for "insn".
 	def dup(self):
@@ -191,6 +197,9 @@ class AwlOperator(object):
 			insn=self.insn)
 
 	def assertType(self, types, lowerLimit=None, upperLimit=None):
+		if self.type == AwlOperator.NAMED_LOCAL or\
+		   self.type == AwlOperator.NAMED_LOCAL_PTR:
+			return #FIXME we should check type for these, too.
 		types = toList(types)
 		if not self.type in types:
 			self._raiseTypeError(self.type, types)
@@ -203,10 +212,24 @@ class AwlOperator(object):
 				raise AwlSimError("Operator value too big",
 						  insn=self.insn)
 
+	# Resolve this indirect operator to a direct operator.
 	def resolve(self, store=True):
 		# This already is a direct operator.
+		if self.type == self.NAMED_LOCAL:
+			# This is a named-local access (#abc).
+			# Resolve it to an interface-operator.
+			return self.insn.cpu.callStackTop.interfRefs[self.interfaceIndex].resolve()
 		return self
 
+	# Make an area-spanning pointer (32 bit) to this memory area.
+	def makePointer(self):
+		try:
+			area = AwlIndirectOp.optype2area[self.type]
+		except KeyError as e:
+			raise AwlSimError("Could not transform operator '%s' "
+				"into a pointer." % str(self))
+		return area | self.value.toPointerValue()
+
 	def __repr__(self):
 		if self.type == self.IMM:
 			if self.width == 8:
@@ -226,7 +249,7 @@ class AwlOperator(object):
 		elif self.type == self.IMM_TOD:
 			return "TOD#" #TODO
 		elif self.type in (self.MEM_A, self.MEM_E,
-				   self.MEM_M, self.MEM_L):
+				   self.MEM_M, self.MEM_L, self.MEM_VL):
 			pfx = self.type2str[self.type]
 			if self.width == 1:
 				return "%s %d.%d" %\
@@ -307,8 +330,6 @@ class AwlOperator(object):
 			return "#%s" % self.value
 		elif self.type == self.NAMED_LOCAL_PTR:
 			return "P##%s" % self.value
-		elif self.type == self.INTERF_DB:
-			return "__INTERFACE_DB" #FIXME
 		elif self.type == self.INDIRECT:
 			assert(0) # Overloaded in AwlIndirectOp
 		elif self.type == self.VIRT_ACCU:
@@ -331,7 +352,7 @@ class AwlIndirectOp(AwlOperator):
 	AR_2		= 2	# Use AR2
 
 	# Address area mask
-	ADDRESS_MASK	= 0x0000FFFFFF
+	ADDRESS_MASK	= 0x0000FFFFFF #FIXME should be 0x7FFFF
 
 	# Pointer area constants
 	AREA_SHIFT	= 24
@@ -356,7 +377,6 @@ class AwlIndirectOp(AwlOperator):
 	EXT_AREA_BLKREF_DB	= 0x03FF000000	# DB block reference
 	EXT_AREA_BLKREF_FB	= 0x04FF000000	# FB block reference
 	EXT_AREA_BLKREF_FC	= 0x05FF000000	# FC block reference
-	EXT_AREA_INTERF_DB	= 0x06FF000000	# INTERF_DB
 
 	# Map for converting area code to operator type for fetch operations
 	area2optype_fetch = {
@@ -373,7 +393,6 @@ class AwlIndirectOp(AwlOperator):
 		EXT_AREA_BLKREF_DB	: AwlOperator.BLKREF_DB,
 		EXT_AREA_BLKREF_FB	: AwlOperator.BLKREF_FB,
 		EXT_AREA_BLKREF_FC	: AwlOperator.BLKREF_FC,
-		EXT_AREA_INTERF_DB	: AwlOperator.INTERF_DB,
 	}
 
 	# Map for converting area code to operator type for store operations
@@ -386,6 +405,20 @@ class AwlIndirectOp(AwlOperator):
 	optype2area[AwlOperator.UNSPEC] = AREA_NONE
 
 	def __init__(self, area, width, addressRegister, offsetOper, insn=None):
+		# area -> The area code for this indirect operation.
+		#         AREA_... or EXT_AREA_...
+		#         This corresponds to the area code in AWL pointer format.
+		# width -> The width (in bits) of the region that is being adressed.
+		# addressRegister -> One of:
+		#                    AR_NONE => This is a memory-indirect access.
+		#                    AR_1 => This is a register-indirect access with AR1.
+		#                    AR_2 => This is a register-indirect access with AR2.
+		# offsetOper -> This is the AwlOperator for the offset.
+		#               For memory-indirect access, this must be an AwlOperator
+		#               with "type in __possibleOffsetOperTypes".
+		#               For register-indirect access, this must be an AwlOperator
+		#               with "type==IMM_PTR".
+		# insn -> The instruction this operator is used in. May be None.
 		AwlOperator.__init__(self,
 				     type = AwlOperator.INDIRECT,
 				     width = width,
@@ -420,8 +453,7 @@ class AwlIndirectOp(AwlOperator):
 	__possibleOffsetOperTypes = (AwlOperator.MEM_M,
 				     AwlOperator.MEM_L,
 				     AwlOperator.MEM_DB,
-				     AwlOperator.MEM_DI,
-				     AwlOperator.INTERF_DB)
+				     AwlOperator.MEM_DI)
 
 	# Resolve this indirect operator to a direct operator.
 	def resolve(self, store=True):
@@ -429,7 +461,7 @@ class AwlIndirectOp(AwlOperator):
 		offsetOper = self.offsetOper
 		# Construct the pointer
 		if self.addressRegister == AwlIndirectOp.AR_NONE:
-			# Indirect access
+			# Memory-indirect access
 			if self.area == AwlIndirectOp.AREA_NONE:
 				raise AwlSimError("Area-spanning access not "
 					"possible in indirect access without "
@@ -491,5 +523,11 @@ class AwlIndirectOp(AwlOperator):
 				(self.width, str(directOffset)))
 		return AwlOperator(optype, self.width, directOffset, self.insn)
 
+	def makePointer(self):
+		# This is a programming error.
+		# The caller should resolve() the operator first.
+		raise AwlSimError("BUG: Can not transform indirect operator "
+			"into a pointer. Resolve it first.")
+
 	def __repr__(self):
 		return "__INDIRECT" #TODO
