@@ -24,6 +24,7 @@ from awlsim.core.compat import *
 
 from awlsim.coreserver.memarea import *
 from awlsim.core.util import *
+from awlsim.core.datatypehelpers import *
 from awlsim.core.cpuspecs import *
 
 import struct
@@ -64,6 +65,7 @@ class AwlSimMessage(object):
 	MSG_ID_CPUSPECS		= EnumGen.item
 	MSG_ID_REQ_MEMORY	= EnumGen.item
 	MSG_ID_MEMORY		= EnumGen.item
+	MSG_ID_INSNSTATE	= EnumGen.item
 	EnumGen.end
 
 	_strLenStruct = struct.Struct(str(">H"))
@@ -273,6 +275,9 @@ class AwlSimMessage_SET_OPT(AwlSimMessage):
 		self.name = name
 		self.value = value
 
+	def getStrValue(self):
+		return self.value
+
 	def getIntValue(self):
 		try:
 			return int(self.value)
@@ -281,7 +286,7 @@ class AwlSimMessage_SET_OPT(AwlSimMessage):
 
 	def getBoolValue(self):
 		try:
-			return bool(self.value)
+			return bool(self.getIntValue())
 		except ValueError as e:
 			raise AwlSimError("SET_OPT: Value is not a boolean")
 
@@ -360,7 +365,6 @@ class AwlSimMessage_CPUSPECS(AwlSimMessage):
 	def __init__(self, cpuspecs):
 		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_CPUSPECS)
 		self.cpuspecs = cpuspecs
-		self.cpuspecs.cpu = None
 
 	def toBytes(self):
 		pl = self.plStruct.pack(self.cpuspecs.getConfiguredMnemonics(),
@@ -456,9 +460,10 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 	#	flags (8 bit)
 	#	index (16 bit)
 	#	start (32 bit)
-	#	length (32 bit)
-	#	the actual binary data (variable length)
-	plAreaStruct = struct.Struct(str(">BBHII"))
+	#	specified length (32 bit)
+	#	actual length (32 bit)
+	#	the actual binary data (variable length, padded to 32-bit boundary)
+	plAreaStruct = struct.Struct(str(">BBHIII"))
 
 	# Flags
 	FLG_SYNC	= 1 << 0 # Synchronous. Returns a REPLY when finished.
@@ -471,12 +476,16 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 	def toBytes(self):
 		pl = [ self.plHdrStruct.pack(self.flags) ]
 		for memArea in self.memAreas:
+			actualLength = len(memArea.data)
 			pl.append(self.plAreaStruct.pack(memArea.memType,
 							 memArea.flags,
 							 memArea.index,
 							 memArea.start,
-							 len(memArea.data)))
-			pl.append(memArea.data) #FIXME padding to 32bit?
+							 memArea.length,
+							 actualLength))
+			pl.append(memArea.data)
+			# Pad to a 32-bit boundary
+			pl.append(b'\x00' * (round_up(actualLength, 4) - actualLength))
 		pl = b''.join(pl)
 		return AwlSimMessage.toBytes(self, len(pl)) + pl
 
@@ -488,13 +497,12 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 			offset += cls.plHdrStruct.size
 			memAreas = []
 			while offset < len(payload):
-				memType, mFlags, index, start, length =\
+				memType, mFlags, index, start, length, actualLength =\
 					cls.plAreaStruct.unpack_from(payload, offset)
 				offset += cls.plAreaStruct.size
-				#FIXME pad length to 32bit?
-				data = payload[offset : offset + length]
-				offset += length
-				if len(data) != length:
+				data = payload[offset : offset + actualLength]
+				offset += round_up(actualLength, 4)
+				if len(data) != actualLength:
 					raise IndexError
 				memAreas.append(MemoryArea(memType, mFlags, index,
 							   start, length, data))
@@ -502,6 +510,48 @@ class AwlSimMessage_MEMORY(AwlSimMessage):
 			raise TransferError("MEMORY: Invalid data format")
 		return cls(flags, memAreas)
 
+class AwlSimMessage_INSNSTATE(AwlSimMessage):
+	# Payload data struct:
+	#	AWL line number (32 bit)
+	#	Serial number. Reset to 0 on cycle exit. (32 bit)
+	#	Flags (16 bit)
+	#	CPU status word (16 bit)
+	#	CPU ACCU 1 (32 bit)
+	#	CPU ACCU 2 (32 bit)
+	#	CPU AR 1 (32 bit)
+	#	CPU AR 2 (32 bit)
+	#	CPU DB register (16 bit)
+	#	CPU DI register (16 bit)
+	plDataStruct = struct.Struct(">IIHHIIIIHH")
+
+	def __init__(self, lineNr, serial, flags, stw, accu1, accu2, ar1, ar2, db, di):
+		AwlSimMessage.__init__(self, AwlSimMessage.MSG_ID_INSNSTATE)
+		self.lineNr = lineNr
+		self.serial = serial
+		self.flags = flags
+		self.stw = stw
+		self.accu1 = accu1
+		self.accu2 = accu2
+		self.ar1 = ar1
+		self.ar2 = ar2
+		self.db = db
+		self.di = di
+
+	def toBytes(self):
+		pl = self.plDataStruct.pack(self.lineNr, self.serial,
+			self.flags, self.stw, self.accu1, self.accu2,
+			self.ar1, self.ar2, self.db, self.di)
+		return AwlSimMessage.toBytes(self, len(pl)) + pl
+
+	@classmethod
+	def fromBytes(cls, payload):
+		try:
+			lineNr, serial, flags, stw, accu1, accu2, ar1, ar2, db, di =\
+				cls.plDataStruct.unpack_from(payload, 0)
+		except (struct.error, IndexError) as e:
+			raise TransferError("INSNSTATE: Invalid data format")
+		return cls(lineNr, serial, flags, stw, accu1, accu2, ar1, ar2, db, di)
+
 class AwlSimMessageTransceiver(object):
 	class RemoteEndDied(Exception): pass
 
@@ -522,10 +572,12 @@ class AwlSimMessageTransceiver(object):
 		AwlSimMessage.MSG_ID_CPUSPECS		: AwlSimMessage_CPUSPECS,
 		AwlSimMessage.MSG_ID_REQ_MEMORY		: AwlSimMessage_REQ_MEMORY,
 		AwlSimMessage.MSG_ID_MEMORY		: AwlSimMessage_MEMORY,
+		AwlSimMessage.MSG_ID_INSNSTATE		: AwlSimMessage_INSNSTATE,
 	}
 
-	def __init__(self, sock):
+	def __init__(self, sock, peerInfoString):
 		self.sock = sock
+		self.peerInfoString = peerInfoString
 
 		# Transmit status
 		self.txSeqCount = 0
@@ -536,7 +588,17 @@ class AwlSimMessageTransceiver(object):
 		self.seq = None
 		self.payloadLen = None
 
-		self.sock.setblocking(False)
+		try:
+			self.__timeout = None
+			self.sock.settimeout(self.__timeout)
+
+			if self.sock.family in (socket.AF_INET, socket.AF_INET6) and\
+			   self.sock.type == socket.SOCK_STREAM:
+				self.sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)
+			self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 2048)
+			self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 8192)
+		except socket.error as e:
+			raise AwlSimError("Failed to initialize socket: %s" % str(e))
 
 	def shutdown(self):
 		if self.sock:
@@ -550,7 +612,11 @@ class AwlSimMessageTransceiver(object):
 				pass
 			self.sock = None
 
-	def send(self, msg):
+	def send(self, msg, timeout=None):
+		if timeout != self.__timeout:
+			self.sock.settimeout(timeout)
+			self.__timeout = timeout
+
 		msg.seq = self.txSeqCount
 		self.txSeqCount = (self.txSeqCount + 1) & 0xFFFF
 
@@ -558,14 +624,28 @@ class AwlSimMessageTransceiver(object):
 		while offset < len(data):
 			try:
 				offset += self.sock.send(data[offset : ])
-			except socket.error as e:
-				if e.errno != errno.EAGAIN:
+			except (socket.error, BlockingIOError) as e:
+				if e.errno != errno.EAGAIN and\
+				   e.errno != errno.EWOULDBLOCK and\
+				   not isinstance(e, BlockingIOError) and\
+				   not isinstance(e, socket.timeout):
 					raise TransferError(str(e))
 
-	def receive(self):
+	def receive(self, timeout=0.0):
+		if timeout != self.__timeout:
+			self.sock.settimeout(timeout)
+			self.__timeout = timeout
+
 		hdrLen = AwlSimMessage.HDR_LENGTH
 		if len(self.buf) < hdrLen:
-			data = self.sock.recv(hdrLen - len(self.buf))
+			try:
+				data = self.sock.recv(hdrLen - len(self.buf))
+			except (socket.error, BlockingIOError) as e:
+				if e.errno == errno.EAGAIN or\
+				   e.errno == errno.EWOULDBLOCK or\
+				   isinstance(e, BlockingIOError):
+					return None
+				raise
 			if not data:
 				# The remote end closed the connection
 				raise self.RemoteEndDied()
@@ -601,13 +681,3 @@ class AwlSimMessageTransceiver(object):
 		msg.seq = self.seq
 		self.buf, self.msgId, self.seq, self.payloadLen = b"", None, None, None
 		return msg
-
-	def receiveBlocking(self, timeoutSec=None):
-		try:
-			self.sock.settimeout(timeoutSec)
-			msg = self.receive()
-		except socket.timeout:
-			return None
-		finally:
-			self.sock.setblocking(False)
-		return msg
