@@ -27,17 +27,22 @@ from awlsim.core.parser import *
 from awlsim.core.cpuspecs import *
 
 from awlsim.coreserver.messages import *
+from awlsim.coreserver.subprocess_wrapper import *
 
 import sys
 import os
 import distutils.spawn
-import subprocess
 import select
 import signal
 import socket
 import errno
 import time
 
+if hasattr(socket, "AF_UNIX"):
+	AF_UNIX = socket.AF_UNIX
+else:
+	AF_UNIX = None
+
 
 class AwlSimServer(object):
 	DEFAULT_HOST	= "localhost"
@@ -54,45 +59,95 @@ class AwlSimServer(object):
 	class Client(object):
 		"""Client information."""
 
-		def __init__(self, sock, host, port):
+		def __init__(self, sock, peerInfoString):
 			# Socket
 			self.socket = sock
-			self.host = host
-			self.port = port
-			self.transceiver = AwlSimMessageTransceiver(sock)
+			self.transceiver = AwlSimMessageTransceiver(sock, peerInfoString)
 
 			# CPU-dump
 			self.dumpInterval = 0
 			self.nextDump = 0
+			# Instruction state dump
+			self.insnStateDump_fromLine = 0
+			self.insnStateDump_toLine = 0
 
 			# Memory read requests
 			self.memReadRequestMsg = None
 			self.repetitionFactor = 0
 			self.repetitionCount = 0
 
+	@classmethod
+	def getaddrinfo(cls, host, port):
+		family, socktype = socket.AF_INET, socket.SOCK_STREAM
+		if os.name == "posix" and host == "localhost" and False: #XXX disabled, for now
+			# We are on posix OS. Instead of AF_INET on localhost,
+			# we use Unix domain sockets.
+			family, socktype = AF_UNIX, socket.SOCK_STREAM
+			sockaddr = "/tmp/awlsim-server-%d.socket" % port
+		else:
+			family, socktype, proto, canonname, sockaddr =\
+				socket.getaddrinfo(host, port, family, socktype)[0]
+		return (family, socktype, sockaddr)
+
+	@classmethod
+	def portIsUnused(cls, host, port):
+		sock = None
+		result = True
+		try:
+			family, socktype, sockaddr = AwlSimServer.getaddrinfo(host, port)
+			if family == AF_UNIX:
+				try:
+					os.stat(sockaddr)
+				except OSError as e:
+					if e.errno == errno.ENOENT:
+						return True
+				return False
+			sock = socket.socket(family, socktype)
+			sock.bind(sockaddr)
+		except socket.error as e:
+			result = False
+		if sock:
+			try:
+				sock.shutdown(socket.SHUT_RDWR)
+			except socket.error as e:
+				pass
+			try:
+				sock.close()
+			except socket.error as e:
+				pass
+		return result
+
+	@classmethod
+	def findExecutable(cls, executable):
+		return distutils.spawn.find_executable(executable)
+
 	@classmethod
 	def start(cls, listenHost, listenPort, forkInterpreter=None):
 		"""Start a new server.
 		If 'forkInterpreter' is not None, spawn a subprocess.
 		If 'forkInterpreter' is None, run the server in this process."""
 
-		environment = {
-			AwlSimServer.ENV_MAGIC		: AwlSimServer.ENV_MAGIC,
-			"AWLSIM_CORESERVER_HOST"	: str(listenHost),
-			"AWLSIM_CORESERVER_PORT"	: str(listenPort),
-			"AWLSIM_CORESERVER_LOGLEVEL"	: str(Logging.getLoglevel()),
-		}
+		# Prepare the environment for the server process.
+		# Inherit from the starter and add awlsim specific variables.
+		env = dict(os.environ)
+		env[AwlSimServer.ENV_MAGIC]		= AwlSimServer.ENV_MAGIC
+		env["AWLSIM_CORESERVER_HOST"]		= str(listenHost)
+		env["AWLSIM_CORESERVER_PORT"]		= str(listenPort)
+		env["AWLSIM_CORESERVER_LOGLEVEL"]	= str(Logging.getLoglevel())
 
 		if forkInterpreter is None:
-			return cls._execute(environment)
+			# Do not fork. Just run the server in this process.
+			return cls._execute(env)
 		else:
-			interp = distutils.spawn.find_executable(forkInterpreter)
+			# Fork a new interpreter process and run server.py as module.
+			interp = cls.findExecutable(forkInterpreter)
+			printInfo("Forking awlsim core server with interpreter '%s'" % interp)
 			if not interp:
 				raise AwlSimError("Failed to find interpreter "
 						  "executable '%s'" % forkInterpreter)
-			serverProcess = subprocess.Popen([interp, "-m", "awlsim.coreserver.server"],
-							 env = environment,
-							 shell = False)
+			serverProcess = PopenWrapper([interp, "-m", "awlsim.coreserver.server"],
+						     env = env,
+						     shell = False)
 			return serverProcess
 
 	@classmethod
@@ -117,9 +172,11 @@ class AwlSimServer(object):
 		return retval
 
 	def __init__(self):
+		self.state = -1
 		self.__setRunState(self.STATE_INIT)
 		self.sim = None
 		self.socket = None
+		self.unixSockPath = None
 		self.clients = []
 
 	def runFromEnvironment(self, env=None):
@@ -153,10 +210,15 @@ class AwlSimServer(object):
 		self.run(host, port)
 
 	def __setRunState(self, runstate):
+		if self.state == self.STATE_EXIT:
+			# We are exiting. Cannot set another state.
+			return
 		self.state = runstate
 		# Make a shortcut variable for RUN
 		self.__running = bool(runstate == self.STATE_RUN)
 
+		self.__insnSerial = 0
+
 	def __rebuildSelectReadList(self):
 		rlist = [ self.socket ]
 		rlist.extend(client.transceiver.sock for client in self.clients)
@@ -171,12 +233,53 @@ class AwlSimServer(object):
 					client.nextDump = now + client.dumpInterval / 1000.0
 					client.transceiver.send(msg)
 
+	def __cpuPostInsnCallback(self, userData):
+		cpu = self.sim.cpu
+		insn = cpu.getCurrentInsn()
+		if not insn:
+			return
+		lineNr, msg = insn.getLineNr(), None
+		for client in self.clients:
+			if client.insnStateDump_fromLine == 0 or\
+			   lineNr < client.insnStateDump_fromLine or\
+			   lineNr > client.insnStateDump_toLine:
+				continue
+			if not msg:
+				msg = AwlSimMessage_INSNSTATE(
+					lineNr & 0xFFFFFFFF,
+					self.__insnSerial,
+					0,
+					cpu.statusWord.getWord(),
+					cpu.accu1.get(),
+					cpu.accu2.get(),
+					cpu.ar1.get(),
+					cpu.ar2.get(),
+					cpu.dbRegister.index & 0xFFFF,
+					cpu.diRegister.index & 0xFFFF)
+			client.transceiver.send(msg)
+		self.__insnSerial += 1
+
+	def __cpuCycleExitCallback(self, userData):
+		self.__insnSerial = 0
+
 	def __updateCpuBlockExitCallback(self):
 		if any(c.dumpInterval for c in self.clients):
 			self.sim.cpu.setBlockExitCallback(self.__cpuBlockExitCallback, None)
 		else:
 			self.sim.cpu.setBlockExitCallback(None)
 
+	def __updateCpuPostInsnCallback(self):
+		if any(c.insnStateDump_fromLine != 0 for c in self.clients):
+			self.sim.cpu.setPostInsnCallback(self.__cpuPostInsnCallback, None)
+		else:
+			self.sim.cpu.setPostInsnCallback(None)
+
+	def __updateCpuCycleExitCallback(self):
+		if any(c.insnStateDump_fromLine != 0 for c in self.clients):
+			self.sim.cpu.setCycleExitCallback(self.__cpuCycleExitCallback, None)
+		else:
+			self.sim.cpu.setCycleExitCallback(None)
+
 	def __rx_PING(self, client, msg):
 		client.transceiver.send(AwlSimMessage_PONG())
 
@@ -243,6 +346,19 @@ class AwlSimServer(object):
 			self.__updateCpuBlockExitCallback()
 		elif msg.name == "cycle_time_limit":
 			self.sim.cpu.setCycleTimeLimit(msg.getFloatValue())
+		elif msg.name == "insn_state_dump":
+			val = msg.getStrValue().split("-")
+			try:
+				if len(val) != 2:
+					raise ValueError
+				fromLine = int(val[0])
+				toLine = int(val[1])
+			except ValueError as e:
+				raise AwlSimError("insn_state_dump: invalid value")
+			client.insnStateDump_fromLine = fromLine
+			client.insnStateDump_toLine = toLine
+			self.__updateCpuPostInsnCallback()
+			self.__updateCpuCycleExitCallback()
 		else:
 			status = AwlSimMessage_REPLY.STAT_FAIL
 
@@ -268,12 +384,21 @@ class AwlSimServer(object):
 
 	def __rx_MEMORY(self, client, msg):
 		cpu = self.sim.cpu
+		status = AwlSimMessage_REPLY.STAT_OK
 		for memArea in msg.memAreas:
-			memArea.writeToCpu(cpu)
+			try:
+				memArea.writeToCpu(cpu)
+			except AwlSimError as e:
+				if memArea.flags & (MemoryArea.FLG_ERR_READ |\
+						    MemoryArea.FLG_ERR_WRITE):
+					# Just signal failure to the client.
+					status = AwlSimMessage_REPLY.STAT_FAIL
+				else:
+					# This is a serious fault.
+					# Re-raise the exception.
+					raise
 		if msg.flags & msg.FLG_SYNC:
-			client.transceiver.send(AwlSimMessage_REPLY.make(
-				msg, AwlSimMessage_REPLY.STAT_OK)
-			)
+			client.transceiver.send(AwlSimMessage_REPLY.make(msg, status))
 
 	__msgRxHandlers = {
 		AwlSimMessage.MSG_ID_PING		: __rx_PING,
@@ -292,16 +417,15 @@ class AwlSimServer(object):
 
 	def __handleClientComm(self, client):
 		try:
-			msg = client.transceiver.receive()
+			msg = client.transceiver.receive(0.0)
 		except AwlSimMessageTransceiver.RemoteEndDied as e:
-			printInfo("AwlSimServer: Client '%s (port %d)' died" %\
-				(client.host, client.port))
+			printInfo("AwlSimServer: Client '%s' died" % client.transceiver.peerInfoString)
 			self.__clientRemove(client)
 			return
 		except (TransferError, socket.error) as e:
-			printInfo("AwlSimServer: Client '%s (port %d)' data "
+			printInfo("AwlSimServer: Client '%s' data "
 				"transfer error:\n%s" %\
-				(client.host, client.port, str(e)))
+				(client.transceiver.peerInfoString, str(e)))
 			return
 		if not msg:
 			return
@@ -337,7 +461,20 @@ class AwlSimServer(object):
 			if client.repetitionCount <= 0:
 				cpu, memAreas = self.sim.cpu, client.memReadRequestMsg.memAreas
 				for memArea in memAreas:
-					memArea.readFromCpu(cpu)
+					memArea.flags = 0
+					try:
+						memArea.readFromCpu(cpu)
+					except AwlSimError as e:
+						if memArea.flags & (MemoryArea.FLG_ERR_READ |\
+								    MemoryArea.FLG_ERR_WRITE):
+							# We do not forward this as an exception.
+							# The client is supposed to check the error bits.
+							# Just continue as usual.
+							pass
+						else:
+							# This is a serious fault.
+							# Re-raise the exception.
+							raise
 				client.transceiver.send(client.memReadRequestMsg)
 				client.repetitionCount = client.repetitionFactor
 				if not client.repetitionFactor:
@@ -364,9 +501,9 @@ class AwlSimServer(object):
 
 				if self.state == self.STATE_RUN:
 					while self.__running:
-						self.__handleCommunication()
 						sim.runCycle()
 						self.__handleMemReadReqs()
+						self.__handleCommunication()
 					continue
 
 			except (AwlSimError, AwlParserError) as e:
@@ -394,10 +531,14 @@ class AwlSimServer(object):
 		"""Listen on 'host':'port'."""
 
 		self.close()
-		printInfo("AwlSimServer: Listening on %s (port %d)..." % (host, port))
 		try:
-			family, socktype, proto, canonname, sockaddr =\
-				socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)[0]
+			family, socktype, sockaddr = AwlSimServer.getaddrinfo(host, port)
+			if family == AF_UNIX:
+				self.unixSockPath = sockaddr
+				readableSockaddr = sockaddr
+			else:
+				readableSockaddr = "%s:%d" % (sockaddr[0], sockaddr[1])
+			printInfo("AwlSimServer: Listening on %s..." % readableSockaddr)
 			sock = socket.socket(family, socktype)
 			sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
 			sock.setblocking(False)
@@ -417,16 +558,19 @@ class AwlSimServer(object):
 
 		try:
 			clientSock, addrInfo = self.socket.accept()
-			clientHost, clientPort = addrInfo[:2]
-		except socket.error as e:
-			if e.errno == errno.EWOULDBLOCK or\
+			if self.unixSockPath:
+				peerInfoString = self.unixSockPath
+			else:
+				peerInfoString = "%s:%d" % addrInfo[:2]
+		except (socket.error, BlockingIOError) as e:
+			if isinstance(e, BlockingIOError) or\
+			   e.errno == errno.EWOULDBLOCK or\
 			   e.errno == errno.EAGAIN:
 				return None
 			raise AwlSimError("AwlSimServer: accept() failed: %s" % str(e))
-		host, port = addrInfo[0], addrInfo[1]
-		printInfo("AwlSimServer: Client '%s (port %d)' connected" % (host, port))
+		printInfo("AwlSimServer: Client '%s' connected" % peerInfoString)
 
-		client = self.Client(clientSock, clientHost, clientPort)
+		client = self.Client(clientSock, peerInfoString)
 		self.__clientAdd(client)
 
 		return client
@@ -465,6 +609,12 @@ class AwlSimServer(object):
 			except socket.error as e:
 				pass
 			self.socket = None
+		if self.unixSockPath:
+			try:
+				os.unlink(self.unixSockPath)
+			except OSError as e:
+				pass
+			self.unixSockPath = None
 
 	def signalHandler(self, sig, frame):
 		printInfo("AwlSimServer: Received signal %d" % sig)
