@@ -2,7 +2,7 @@
 #
 # AWL simulator - GUI CPU widget
 #
-# Copyright 2012-2013 Michael Buesch <m@bues.ch>
+# Copyright 2012-2014 Michael Buesch <m@bues.ch>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,8 +23,8 @@ from __future__ import division, absolute_import, print_function, unicode_litera
 from awlsim.core.compat import *
 
 from awlsim.gui.util import *
-from awlsim.gui.pseudohardware import *
 from awlsim.gui.cpustate import *
+from awlsim.gui.awlsimclient import *
 
 
 class CpuWidget(QWidget):
@@ -32,7 +32,6 @@ class CpuWidget(QWidget):
 
 	EnumGen.start
 	STATE_STOP	= EnumGen.item
-	STATE_PARSE	= EnumGen.item
 	STATE_INIT	= EnumGen.item
 	STATE_LOAD	= EnumGen.item
 	STATE_RUN	= EnumGen.item
@@ -43,13 +42,14 @@ class CpuWidget(QWidget):
 		self.setLayout(QGridLayout(self))
 
 		self.mainWidget = mainWidget
-		self.sim = mainWidget.getSim()
 		self.state = self.STATE_STOP
-		self.__nextCpuWidgetUpdate = 0.0
 
-		self.pseudoHw = GuiPseudoHardwareInterface(sim = self.sim,
-							   cpuWidget = self)
-		self.sim.registerHardware(self.pseudoHw)
+		client = self.mainWidget.getSimClient()
+		client.haveCpuDump.connect(self.__handleCpuDump)
+		client.haveInsnDump.connect(self.mainWidget.codeEdit.updateCpuStats_afterInsn)
+		client.haveMemoryUpdate.connect(self.__handleMemoryUpdate)
+
+		self.mainWidget.codeEdit.visibleRangeChanged.connect(self.__updateOnlineViewState)
 
 		group = QGroupBox("CPU status", self)
 		group.setLayout(QGridLayout(group))
@@ -98,115 +98,144 @@ class CpuWidget(QWidget):
 
 	def __addWindow(self, win):
 		self.stateWs.addWindow(win, Qt.Window)
+		win.configChanged.connect(self.__stateWinConfigChanged)
 		win.show()
 		self.update()
+		self.__uploadMemReadAreas()
 
 	def __newWin_CPU(self):
-		self.__addWindow(State_CPU(self.mainWidget.getSim(), self))
+		self.__addWindow(State_CPU(self.mainWidget.getSimClient(), self))
 
 	def __newWin_DB(self):
-		self.__addWindow(State_Mem(self.mainWidget.getSim(),
+		self.__addWindow(State_Mem(self.mainWidget.getSimClient(),
 					   AbstractDisplayWidget.ADDRSPACE_DB,
 					   self))
 
 	def __newWin_E(self):
-		self.__addWindow(State_Mem(self.mainWidget.getSim(),
+		self.__addWindow(State_Mem(self.mainWidget.getSimClient(),
 					   AbstractDisplayWidget.ADDRSPACE_E,
 					   self))
 
 	def __newWin_A(self):
-		self.__addWindow(State_Mem(self.mainWidget.getSim(),
+		self.__addWindow(State_Mem(self.mainWidget.getSimClient(),
 					   AbstractDisplayWidget.ADDRSPACE_A,
 					   self))
 
 	def __newWin_M(self):
-		self.__addWindow(State_Mem(self.mainWidget.getSim(),
+		self.__addWindow(State_Mem(self.mainWidget.getSimClient(),
 					   AbstractDisplayWidget.ADDRSPACE_M,
 					   self))
 
 	def __newWin_LCD(self):
-		self.__addWindow(State_LCD(self.mainWidget.getSim(), self))
+		self.__addWindow(State_LCD(self.mainWidget.getSimClient(), self))
+
+	def __stateWinConfigChanged(self, stateWin):
+		self.__uploadMemReadAreas()
 
 	def update(self):
 		for win in self.stateWs.windowList():
 			win.update()
 
-	# Get the queued CPU store requests
-	def getQueuedStoreRequests(self):
-		reqList = []
+	# Upload the used memory area descriptors to the core.
+	def __uploadMemReadAreas(self):
+		client = self.mainWidget.getSimClient()
+		wantDump = False
+		memAreas = []
 		for win in self.stateWs.windowList():
-			reqList.extend(win.getQueuedStoreRequests())
-		return reqList
+			memAreas.extend(win.getMemoryAreas())
+			if isinstance(win, State_CPU):
+				wantDump = True
+		try:
+			client.setMemoryReadRequests(memAreas,
+						     repetitionFactor = 10,
+						     sync = True)
+			client.setPeriodicDumpInterval(300 if wantDump else 0)
+		except AwlSimError as e:
+			MessageBox.handleAwlSimError(self,
+				"Communication error with the simulator core",
+				e)
 
-	def __cycleExitCallback(self, cpu):
-		if self.state == self.STATE_RUN:
-			self.mainWidget.codeEdit.updateCpuStats_afterCycle(cpu)
+	def __handleCpuDump(self, dumpText):
+		for win in self.stateWs.windowList():
+			if isinstance(win, State_CPU):
+				win.setDumpText(dumpText)
 
-	def __blockExitCallback(self, cpu):
-		if self.state == self.STATE_RUN:
-			self.mainWidget.codeEdit.updateCpuStats_afterBlock(cpu)
-
-			# Special case: May update the CPU-state-widgets (if any)
-			# on block exit.
-			if cpu.now >= self.__nextCpuWidgetUpdate:
-				self.__nextCpuWidgetUpdate = cpu.now + 0.15
-				for win in self.stateWs.windowList():
-					if isinstance(win, State_CPU):
-						win.update()
-
-	def __postInsnCallback(self, cpu):
-		if self.state == self.STATE_RUN:
-			self.mainWidget.codeEdit.updateCpuStats_afterInsn(cpu)
-
-	def __screenUpdateCallback(self, cpu):
-		self.__postInsnCallback(cpu)
-		self.__blockExitCallback(cpu)
-		self.__cycleExitCallback(cpu)
-		QApplication.processEvents(QEventLoop.AllEvents, 100)
+	def __handleMemoryUpdate(self, memAreas):
+		for win in self.stateWs.windowList():
+			win.setMemories(memAreas)
 
 	def __run(self):
-		sim = self.mainWidget.getSim()
-		self.__updateOnlineViewState()
-		self.__setState(self.STATE_PARSE)
+		client = self.mainWidget.getSimClient()
+
+		self.__setState(self.STATE_INIT)
 		self.runButton.setChecked(True)
 		self.runButton.setEnabled(False) # Redraws the radio button
 		self.runButton.setEnabled(True)
+
 		ob1_awl = self.mainWidget.getCodeEditWidget().getCode()
 		if not ob1_awl.strip():
 			MessageBox.error(self, "No AWL/STL code available. Cannot run.")
 			self.stop()
 			return
+
 		try:
-			parser = AwlParser()
-			parser.parseData(ob1_awl)
-			self.__setState(self.STATE_INIT)
-			cpu = sim.getCPU()
-			cpu.setBlockExitCallback(self.__blockExitCallback, cpu)
-			cpu.setCycleExitCallback(self.__cycleExitCallback, cpu)
-			cpu.setScreenUpdateCallback(
-				self.__screenUpdateCallback, cpu)
+			if self.mainWidget.coreConfigDialog.shouldSpawnServer():
+				firstPort, lastPort = self.mainWidget.coreConfigDialog.getSpawnPortRange()
+				interp = self.mainWidget.coreConfigDialog.getInterpreterList()
+				host = AwlSimServer.DEFAULT_HOST
+				for port in range(firstPort, lastPort + 1):
+					if not AwlSimServer.portIsUnused(host, port):
+						continue
+					client.spawnServer(interpreter = interp,
+							   listenHost = host,
+							   listenPort = port)
+					break
+				else:
+					raise AwlSimError("Did not find a free port to run the "
+						"awlsim core server on.\nTried port %d to %d on '%s'." %\
+						(firstPort, lastPort, host))
+			else:
+				host = self.mainWidget.coreConfigDialog.getConnectHost()
+				port = self.mainWidget.coreConfigDialog.getConnectPort()
+			client.connectToServer(host = host,
+					       port = port)
+			client.setRunState(False)
+			client.reset()
+
+			self.mainWidget.cpuConfigDialog.uploadToCPU()
+			self.__uploadMemReadAreas()
+			self.__updateOnlineViewState()
+
 			self.__setState(self.STATE_LOAD)
-			sim.reset()
-			sim.load(parser.getParseTree())
-			sim.startup()
+			client.loadHardwareModule("dummy")
+			client.loadCode(ob1_awl)
+			client.setRunState(True)
 		except AwlParserError as e:
 			MessageBox.handleAwlParserError(self, e)
 			self.stop()
-			sim.shutdown()
+			client.shutdown()
 			return
 		except AwlSimError as e:
 			MessageBox.handleAwlSimError(self,
 				"Error while loading code", e)
 			self.stop()
-			sim.shutdown()
+			client.shutdown()
 			return
 		except Exception:
+			try:
+				client.setRunState(False)
+			except: pass
+			client.shutdown()
 			handleFatalException(self)
 		self.__setState(self.STATE_RUN)
 		try:
+			# The main loop
 			while self.state == self.STATE_RUN:
-				sim.runCycle()
-				QApplication.processEvents(QEventLoop.AllEvents, 100)
+				# Receive messages, until we hit a timeout
+				while client.processMessages(0.1):
+					pass
+				# Process GUI events
+				QApplication.processEvents(QEventLoop.AllEvents)
 		except AwlSimError as e:
 			MessageBox.handleAwlSimError(self,
 				"Error while executing code", e)
@@ -214,14 +243,21 @@ class CpuWidget(QWidget):
 		except MaintenanceRequest as e:
 			if e.requestType == MaintenanceRequest.TYPE_SHUTDOWN:
 				print("Shutting down, as requested...")
-				sim.shutdown()
+				client.shutdown()
 				QApplication.exit(0)
 				return
 			else:
 				assert(0)
 		except Exception:
+			try:
+				client.setRunState(False)
+			except: pass
+			client.shutdown()
 			handleFatalException(self)
-		sim.shutdown()
+		try:
+			client.setRunState(False)
+		except: pass
+		client.shutdown()
 
 	def stop(self):
 		if self.state == self.STATE_STOP:
@@ -255,10 +291,14 @@ class CpuWidget(QWidget):
 	def __updateOnlineViewState(self):
 		en = self.onlineViewCheckBox.checkState() == Qt.Checked
 		self.mainWidget.codeEdit.enableCpuStats(en)
-		cpu = self.mainWidget.getSim().getCPU()
-		if en:
-			cpu.setPostInsnCallback(self.__postInsnCallback, cpu)
-		else:
-			cpu.setPostInsnCallback(None)
-
-
+		try:
+			client = self.mainWidget.getSimClient()
+			if en:
+				fromLine, toLine = self.mainWidget.codeEdit.getVisibleLineRange()
+				client.setInsnStateDump(fromLine, toLine, sync=False)
+			else:
+				client.setInsnStateDump(0, 0, sync=False)
+		except AwlSimError as e:
+			MessageBox.handleAwlSimError(self,
+				"Failed to setup instruction dumping", e)
+			return
