@@ -1,8 +1,12 @@
 #!/bin/sh
 
+# basedir is the root of the test directory in the package
 basedir="$(dirname "$0")"
 [ "$(echo "$basedir" | cut -c1)" = '/' ] || basedir="$PWD/$basedir"
 
+# rootdir is the root of the package
+rootdir="$basedir/.."
+
 die()
 {
 	echo "$*"
@@ -23,11 +27,11 @@ get_interpreter_version()
 {
 	local interpreter="$1"
 
-	"$interpreter" -c 'import sys; print("%d %d %d" % sys.version_info[0:3]);'
+	"$interpreter" -c 'import sys; print("%d %d %d" % sys.version_info[0:3]);' 2>/dev/null
 }
 
-# $1=interpreter $2=awl_file ($3..$x additional options to awlsimcli)
-run_test()
+# $1=interpreter $2=awl_file ($3ff additional options to awlsimcli)
+run_awl_test()
 {
 	local interpreter="$1"
 	local awl="$2"
@@ -40,15 +44,59 @@ run_test()
 	local awlpro="${dir}/${base}.awlpro"
 	[ -r "$awlpro" ] && awl="$awlpro"
 
-	echo -n "Running test '$(basename "$awl")' ..."
 	command time -o "$test_time_file" -f '%E' \
-	"$interpreter" "$basedir/../awlsimcli" --quiet --onecycle --extended-insns \
-		--hardware dummy:inputAddressBase=7:outputAddressBase=8:dummyParam=True \
+	"$interpreter" "$rootdir/awlsimcli" --quiet --onecycle --extended-insns \
+		--hardware debug:inputAddressBase=7:outputAddressBase=8:dummyParam=True \
 		--cycle-time 60 \
 		"$@" \
 		"$awl" ||\
 		die "Test failed"
-	echo " [$(cat "$test_time_file")]"
+	echo "[OK: $(cat "$test_time_file")]"
+}
+
+# $1=interpreter $2=sh_file
+run_sh_test()
+{
+	local interpreter="$1"
+	local sh_file="$2"
+	shift; shift
+
+	[ -x "$sh_file" ] && die "SH-file '$sh_file' must NOT be executable"
+
+	[ "$(echo "$sh_file" | cut -c1)" = '/' ] || local sh_file="./$sh_file"
+
+	# Source the test file
+	. "$basedir/sh-test.defaults"
+	. "$sh_file"
+
+	# Run the test
+	( sh_test "$interpreter" )
+	local result=$?
+
+	[ $result -eq 0 ] || die "Test failed with error code $result"
+	echo "[OK]"
+}
+
+# $1=interpreter $2=testfile(.awl/.sh) ($3ff additional options to awlsimcli or testfile)
+run_test()
+{
+	local interpreter="$1"
+	local testfile="$2"
+	shift; shift
+
+	# Don't run ourself
+	[ "$(basename "$testfile")" = "run.sh" ] && return
+
+	echo -n "Running test '$(basename "$testfile")' with '$(basename "$interpreter")' ... "
+
+	# Check the file type and run the tester
+	if [ "$(echo -n "$testfile" | tail -c4)" = ".awl" ]; then
+		run_awl_test "$interpreter" "$testfile" "$@"
+	elif [ "$(echo -n "$testfile" | tail -c3)" = ".sh" ]; then
+		run_sh_test "$interpreter" "$testfile" "$@"
+	else
+		die "Test file type of '$testfile' not recognized"
+	fi
 }
 
 # $1=interpreter, $2=directory
@@ -58,11 +106,19 @@ run_test_directory()
 	local directory="$2"
 
 	echo "--- Entering directory '$directory'"
+	# run .awl(pro) tests
 	for entry in "$directory"/*; do
 		[ -d "$entry" ] && continue
 		[ "$(echo -n "$entry" | tail -c4)" = ".awl" ] || continue
 		run_test "$interpreter" "$entry"
 	done
+	# run .sh tests
+	for entry in "$directory"/*; do
+		[ -d "$entry" ] && continue
+		[ "$(echo -n "$entry" | tail -c3)" = ".sh" ] || continue
+		run_test "$interpreter" "$entry"
+	done
+	# Recurse into subdirectories
 	for entry in "$directory"/*; do
 		[ -d "$entry" ] || continue
 		run_test_directory "$interpreter" "$entry"
@@ -70,24 +126,10 @@ run_test_directory()
 	echo "--- Leaving directory '$directory'"
 }
 
-# Run coreserver tests.
-# $1=interpreter
-run_server_tests()
-{
-	local interpreter="$1"
-
-	echo "--- Running coreserver tests"
-	for testfile in shutdown.awl; do
-		run_test "$interpreter" "$basedir/$testfile" \
-			--spawn-backend --interpreter "$interpreter"
-	done
-	echo "--- Finished coreserver tests"
-}
-
 # $@=testfiles
 do_tests()
 {
-	for interpreter in "$opt_interpreter" python2 python3 pypy pypy3 jython; do
+	for interpreter in "$opt_interpreter" python2 python3 pypy pypy3 jython ipy; do
 		[ -z "$interpreter" ] && continue
 		which "$interpreter" >/dev/null 2>&1 || {
 			echo "=== WARNING: '$interpreter' interpreter not found. Test skipped."
@@ -109,7 +151,6 @@ do_tests()
 		echo "=== Running tests with '$interpreter' interpreter."
 		if [ $# -eq 0 ]; then
 			run_test_directory "$interpreter" "$basedir"
-			run_server_tests "$interpreter"
 		else
 			for opt in "$@"; do
 				if [ -d "$opt" ]; then
